# Copyright (C) 2000 NEC Corporation
# All Rights Reserved.

# RCS: $Id: namedapi.pl,v 1.1.1.1 2002/09/09 07:43:35 snavi Exp $


#---------------------------------------------------------------------------
require "/opt/nec/wbmc/lib/dns/named.pl";

#---------------------------------------------------------------------------
# 255.255.255.0 -> 24
#---------------------------------------------------------------------------
sub mask2bit {
	local($mask) = @_;

	my ($bit, $b, $error, $end_flag, $counter, $cnt);

	$bit = 0;
	$error = 0;
	$end_flag = 0;
	$counter = 0;
	$cnt = 0;

	if (!&checkNetMask($mask)) {
		return 0;
	}
	@col = split(/\./,$mask);

	while ($counter < 4) {
		$b = 128;
		for ($cnt = 0 ; $cnt < 8 ; $cnt ++) {
			if(($col [$counter] <= 255) 
				&& ($col[$counter] & $b)) { 
				if ($end_flag){
					$error++;
					last;
				}
				$bit++;
			} else {
				$end_flag = 1;
			}
			$b = $b >> 1;
		}
		$counter++;
	}
	if ($error != 0) {
		return 0;
	}
	return $bit;
}

#---------------------------------------------------------------------------
# 24 -> 255.255.255.0
#---------------------------------------------------------------------------
sub bit2mask {
	local($bit) = @_;
	my ($mask, $m, $i);

	if (!&checkNum($bit, 1, 31)) {
		return 0;
	}
	$m = 0;
	for ($i = 0; $i < $bit; $i++) {
		$m += (1 << (31 - $i));
	}
	$mask = &num2Addr($m);
	return $mask;
}

#---------------------------------------------------------------------------
# 99.8.10.in-addr.arpa. -> 10.8.99.0/24
# 0/25.1.168.192.in-addr.arpa. -> 192.168.1.0/25
#---------------------------------------------------------------------------
sub reverse2Net {
	local ($val) = @_;
	my ($net, $mask, @col, @col2, $i);

	$net = &getAddr($val);
	if (!$net) {
		return 0;
	}
	@col = split(/\./, $net);
	if ($#col < 0) {
		return 0;
	}
	@col2 = split(/\//, $col[$#col]);
	if ($#col2 != 0 && $#col2 != 1) {
		return 0;
	}
	if ($#col2 == 1) {
		$net = "";
		for ($i = 0; $i < $#col; $i++) {
			if ($i != 0) {
				$net .= "\.";
			}
			$net .= $col[$i];
		}
		$net .= "\.$col2[0]";
	}
	for ($i = $#col; $i < 3; $i++) {
		$net .= "\.";
		$net .= "0";
	}
	if ($#col2 == 0) {
		if ($#col < 3) {
			$mask = 8 * ($#col + 1);
		}
		else {
			$mask = 24;
			for ($i = 0; $i < 7; $i++) {
				$tmpval = ($col2[0] >> $i);
				if (($tmpval & 1)) {
					$mask++;
					last;
				}
				$mask++;
			}
		}
		if (!&checkNum($mask, 1, 31)) {
			return 0;
		}
		$net .= "/$mask";
	}
	else {
		if (!&checkNum($col2[1], 1, 31)) {
			return 0;
		}
		$net .= "/$col2[1]";
	}
	return $net;
}

#---------------------------------------------------------------------------
# dv4.nnes.nec.co.jp -> dv4.nnes.nec.co.jp.
#---------------------------------------------------------------------------
sub getOrigin {
	local($val) = @_;
	my ($preorg, $len, $w);

	$preorg = $val;
	$len = length($preorg);

	$w = substr($preorg, $len-1, 1);
	if ($w ne ".") {
		$preorg .= ".";
	}
	return $preorg;
}

#---------------------------------------------------------------------------
# 10.8.99.0/24 -> 99.8.10
#---------------------------------------------------------------------------
sub getRnet {
	local($val) = @_;
	my ($rnetwork, @a, @b, @c, $mask, $bit, $maskdisp, $i);
	
	$rnetwork = "";
	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}

	$maskdisp = 0;
	$bit = $a[1];
	if ($bit % 8) {
		$maskdisp = 1;
	}

	@b = split(/\./, $a[0]);
	if ($#b < 3) {
		return 0;
	}

	if (!($mask = bit2mask($a[1]))) {
		return 0;
	}
	@a = split(/\./, $mask);
	if ($#a < 3) {
		return 0;
	}

	for ($i = 0; $i <= $#a; $i++) {
		if ($a[$i] == 0) {
			last;
		}
		$c[$i] = $a[$i] & $b[$i];
	}

	for ($i = $#c; $i >= 0; $i--) {
		if ($i != $#c) {
			$rnetwork .= ".";
		}
		$rnetwork .= $c[$i];
		if (($i == $#c) && ($maskdisp == 1)) {
			$rnetwork .= "\/$bit";
		}
	}
	return $rnetwork;
}

#---------------------------------------------------------------------------
# 10.8.99.0/24 -> 99.8.10.IN-ADDR.ARPA
#---------------------------------------------------------------------------
sub net2Zone {
	local($val) = @_;
	my ($rnetwork, @a, @b, @c, $mask, $bit, $maskdisp, $i);
	
	$rnetwork = "";
	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}
	$maskdisp = 0;
	$bit = $a[1];
	if ($bit % 8) {
		$maskdisp = 1;
	}

	@b = split(/\./, $a[0]);
	if ($#b < 3) {
		return 0;
	}

	if (!($mask = bit2mask($a[1]))) {
		return 0;
	}
	@a = split(/\./, $mask);
	if ($#a < 3) {
		return 0;
	}

	for ($i = 0; $i <= $#a; $i++) {
		if ($a[$i] == 0) {
			last;
		}
		$c[$i] = $a[$i] & $b[$i];
		if ($c[$i] != $b[$i]) {
			return 0;
		}
	}


	for ($i = $#c; $i >= 0; $i--) {
		if ($i != $#c) {
			$rnetwork .= ".";
		}
		$rnetwork .= $c[$i];
		if (($i == $#c) && ($maskdisp == 1)) {
			$rnetwork .= "\/$bit";
		}
	}
	$rnetwork .= "\.IN\-ADDR\.ARPA";
	return $rnetwork;
}

#---------------------------------------------------------------------------
# 10.8.99.0/24 -> 10.8.99
#---------------------------------------------------------------------------
sub getNet {
	local($val) = @_;
	my ($network, @a, @b, $c, $mask, $i, $rnetwork);
	
	$rnetwork = "";
	$network = "";

	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}
	@b = split(/\./, $a[0]);
	if ($#a > 3) {
		return 0;
	}
	if (!($mask = bit2mask($a[1]))) {
		return 0;
	}
	@a = split(/\./, $mask);
	if ($#a < 3) {
		return 0;
	}
	for ($i = 0; $i <= $#a; $i++) {
		if ($a[$i] == 0) {
			last;
		}
		if ($i != 0) {
			$network .= ".";
		}
		$network .= $a[$i] & $b[$i];
	}
	return $network;
}
sub getShortNet {
	local($val) = @_;
	my ($network, @a, @b, $c, $mask, $i, $rnetwork);
	
	$rnetwork = "";
	$network = "";

	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}
	@b = split(/\./, $a[0]);
	if ($#a > 3) {
		return 0;
	}
	if (!($mask = bit2mask($a[1]))) {
		return 0;
	}
	@a = split(/\./, $mask);
	if ($#a < 3) {
		return 0;
	}
	for ($i = 0; $i <= $#a; $i++) {
		if ($a[$i] != 255) {
			last;
		}
		if ($i != 0) {
			$network .= ".";
		}
		$network .= $a[$i] & $b[$i];
	}
	return $network;
}

#---------------------------------------------------------------------------
# 10.8.99.0/24 -> 10.8.99.0
#---------------------------------------------------------------------------
sub getFullNet {
	local($val) = @_;
	my ($network, @a, @b, $c, $mask, $i);
	
	$network = "";

	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}
	@b = split(/\./, $a[0]);
	if ($#a > 3) {
		return 0;
	}
	if (!($mask = bit2mask($a[1]))) {
		return 0;
	}
	@a = split(/\./, $mask);
	if ($#a < 3) {
		return 0;
	}
	for ($i = 0; $i <= $#a; $i++) {
		if ($i != 0) {
			$network .= ".";
		}
		if ($a[$i] == 0) {
			if ($b[$i] != 0) {
				return 0;
			}
			$network .= "0";
		}
		else {
			$network .= $a[$i] & $b[$i];
		}
	}
	return $network;
}

#---------------------------------------------------------------------------
# 10.8.99.0/24 -> 256
#---------------------------------------------------------------------------
sub getHnum {
	local($val) = @_;
	my ($hostnum, @a, $i, $max);
	
	$hostnum = 0;
	@a = split(/\//, $val);
	if ($#a != 1) {
		return 0;
	}
	if (!&ipAddrCheck($a[0]) || !&checkNum($a[1], 1, 31)) {
		return 0;
	}
	$max = 32 - $a[1];

	for ($i = 0; $i <= $max; $i++) {
		if ($i == 0) {
			$hostnum = 1;
		}
		else {
			$hostnum *= 2;
		}
	}
	return $hostnum;
}

#---------------------------------------------------------------------------
# 65534 -> 255.254
#---------------------------------------------------------------------------
sub num2Addr {
	local($val) = @_;
	my ($hostaddr, @n, $i, $atfirst);

	if (!&checkNum($val, 0, 4294967295)) {
		return 0;
	}
	$hostaddr = "";
	$n[0] = ($val >> 24) & 0xff;
	$n[1] = ($val >> 16) & 0xff;
	$n[2] = ($val >> 8) & 0xff;
	$n[3] = $val & 0xff;

	$atfirst = 1;
	for ($i = 0; $i <= $#n; $i++) {
		if (($atfirst == 1) && !$n[$i]) {
			next;
		}
		if ($atfirst == 0) {
			$hostaddr .= ".";
		}
		$hostaddr .= "$n[$i]";
		$atfirst = 0;
	}
	if ($atfirst == 1) {
		$hostaddr = "0";
	}
	return $hostaddr;
}

#---------------------------------------------------------------------------
# 65534 -> 0.0.255.254
#---------------------------------------------------------------------------
sub num2FullAddr {
	local($val) = @_;
	my ($addr, $fulladdr, @col, $atfirst, $i, $n);
	
	if (!&checkNum($val, 0, 4294967295)) {
		return 0;
	}
	$fulladdr = "";
	$addr = &num2Addr($val);
	@col = split(/\./, $addr);
	$n = 3 - $#col;
	$atfirst = 1;
	for ($i = 0; $i < $n; $i++) {
		if ($i > 0) {
			$fulladdr .= ".";
		}
		$fulladdr .= "0";
		$atfirst = 0;
	}
	for ($i = 0; $i <= $#col; $i++) {
		if ($atfirst == 0) {
			$fulladdr .= ".";
		}
		$fulladdr .= "$col[$i]";
		$atfirst = 0;
	}
	return $fulladdr;
}

#---------------------------------------------------------------------------
# 0.0.255.254 -> 65534
#---------------------------------------------------------------------------
sub addr2Num {
	local($val) = @_;
	my (@col, $i, $num);

	if (!&ipAddrCheck($val)) {
		return 0;
	}
	@col = split(/\./, $val);
	if ($#col != 3) {
		return 0;
	}
	$num = 0;
	$num += ($col[0] << 24);
	$num += ($col[1] << 16);
	$num += ($col[2] << 8);
	$num += $col[3];

	return $num;
}

#---------------------------------------------------------------------------
# 65534 -> 254.255
#---------------------------------------------------------------------------
sub num2Raddr {
	local($val) = @_;
	my ($hostraddr, @n, $i, $hostaddr, $atfirst);

	if (!&checkNum($val, 0, 4294967295)) {
		return 0;
	}
	$hostaddr = "";
	$hostraddr = "";
	$n[0] = ($val >> 24) & 0xff;
	$n[1] = ($val >> 16) & 0xff;
	$n[2] = ($val >> 8) & 0xff;
	$n[3] = $val & 0xff;

	$atfirst = 1;
	for ($i = 0; $i <= $#n; $i++) {
		if (($atfirst == 1) && !$n[$i]) {
			next;
		}
		if ($atfirst == 0) {
			$hostaddr .= ".";
		}
		$hostaddr .= $n[$i];
		$atfirst = 0;
	}

	@n = split(/\./, $hostaddr);
	$atfirst = 1;
	for ($i = $#n; $i >= 0; $i--) {
		if ($atfirst == 0) {
			$hostraddr .= ".";
		}
		$hostraddr .= $n[$i];
		$atfirst = 0;
	}
	if ($atfirst == 1) {
		$hostaddr = "0";
	}

	return $hostraddr;
}

#---------------------------------------------------------------------------
# 99.8.10.in-addr.arpa. 100          -> 10.8.99.100
# 99.8.10.in-addr.arpa. 100.99.8.10. -> 10.8.99.100
#---------------------------------------------------------------------------
sub getFullAddr {
	local($origin, $domain) = @_;

	my ($fulladdr, $w, $val, $tmpval, @col, @col2, $i, $len);

	$len = length($domain);
	$w = substr($domain, $len-1, 1);
	$fulladdr = "";
	if ($w ne ".") {
		$tmpval = $origin;
		$tmpval =~ s/\.IN\-ADDR\.ARPA[\.]?//i;
		if ("$origin" eq "$tmpval") {
			return 0;
		}
		@col = split(/\./, $tmpval);
		if ($#col < 0) {
			return 0;
		}
		for ($i = $#col; $i >= 0; $i--) {
			@col2 = split(/\//, $col[$i]);
			if ($#col2 > 0) {
				next;
			}
			if ($i != $#col) {
				$fulladdr .= ".";
			}
			$fulladdr .= "$col[$i]";
		}
		@col = split(/\./, $domain);
		for ($i = $#col; $i >= 0; $i--) {
			$fulladdr .= ".";
			$fulladdr .= $col[$i];
		}
	}
	else {
		$tmpval = $domain;
		$tmpval =~ s/\.IN\-ADDR\.ARPA[\.]?//i;
		if ("$domain" eq "$tmpval") {
			return 0;
		}
		@col = split(/\./, $tmpval);
		if ($#col < 0) {
			return 0;
		}
		for ($i = $#col; $i >= 0; $i--) {
			@col2 = split(/\//, $col[$i]);
			if ($#col2 > 0) {
				next;
			}
			if ($i != $#col) {
				$fulladdr .= ".";
			}
			$fulladdr .= "$col[$i]";
		}
	}
	if (!&ipAddrCheck($fulladdr)) {
		return 0;
	}
	return $fulladdr;
}
#---------------------------------------------------------------------------
# dv4.nnes.nec.co.jp. host1  -> host1.dv4.nnes.nec.co.jp.
# dv4.nnes.nec.co.jp. host1. -> host1.
#---------------------------------------------------------------------------
sub getFullDomain {
	local($origin, $domain) = @_;
	my ($fulldomain, $w, $len);

	$len = length($domain);
	$w = substr($domain, $len-1, 1);
	if ($w ne ".") {
		$fulldomain = "$domain.$origin";
	}
	else {
		$fulldomain = "$origin";
	}
	return $fulldomain;
}
#---------------------------------------------------------------------------
# 99.8.10.in-addr.arpa. -> 10.8.99
#---------------------------------------------------------------------------
sub getAddr {
	local($val) = @_;
	my ($addr, $tmpval, @col, @col2, $i);

	$tmpval = $val;
	$tmpval =~ s/\.IN\-ADDR\.ARPA[\.]?//i;
	if ("$val" eq "$tmpval") {
		return 0;
	}
	@col = split(/\./, $tmpval);
	if ($#col < 0 || $#col > 3) {
		return 0;
	}
	$addr = "";
	for ($i = $#col; $i >= 0; $i--) {
		if ($i == 0) {
			@col2 = split(/\//, $col[0]);
			if ($#col2 != 0 && $#col2 != 1) {
				return 0;
			}
			if (!&checkNum($col2[0], 0, 255)) {
				return 0;
			}
			if ($#col2 == 1) {
				if (!&checkNum($col2[1], 1, 31)) {
					return 0;
				}
			}
		}
		else {
			if (!&checkNum($col[$i], 0, 255)) {
				return 0;
			}
		}
		if ($i != $#col) {
			$addr .= ".";
		}
		$addr .= "$col[$i]";
	}
	return $addr;

}

#---------------------------------------------------------------------------
# 99.8.10 -> 10.8.99
#---------------------------------------------------------------------------
sub reverseAddr {
	local($val) = @_;
	my (@col, $i, @col2, $addr);

	@col = split(/\./, $val);
	if ($#col < 0 || $#col > 3) {
		return 0;
	}
	for ($i = $#col; $i >= 0; $i--) {
		if ($i == 0) {
			@col2 = split(/\//, $col[0]);
			if ($#col2 != 0 && $#con2 != 1) {
				return 0;
			}
			if (!&checkNum($col2[0], 0, 255)) {
				return 0;
			}
			if ($#col2 == 1) {
				if (!&checkNum($col2[1], 1, 31)) {
					return 0;
				}
			}
		}
		else {
			if (!&checkNum($col[$i], 0, 255)) {
				return 0;
			}
		}
		if ($i != $#col) {
			$addr .= ".";
		}
		$addr .= $col[$i];
	}
	return $addr;
}

#---------------------------------------------------------------------------
sub parseTextareaArg {
	local($buf, $keywd, *taa) = @_;
	my(@col1, @col2, $argc, $i, $j);

	if ($buf) {
		$buf =~ s/^\s*//;
		@col1 = split(/[\s,]+/, $buf);
		$argc = 0;
		$taa->{"$keywd"}->{"argc"} = $argc;
		for ($i = 0; $i <= $#col1; $i++) {
			$taa->{"$keywd"}->{"argc"} = $argc+1;
			$taa->{"$keywd"}->{$argc} = $col1[$i];
			$argc++;
		}
	}
}

sub parseTextarea {
	local($buf, *ta) = @_;
	my(@val, @col1, @col2, $i, $j, $k, $err);

	$err = 0;
	if ($buf) {
		$k = 0;
		$buf =~ s/^\s*//;
		@col1 = split(/[\s,]+/, $buf);
		for ($i = 0; $i <= $#col1; $i++) {
			$val[$k] = $col1[$i];
			$k++;
		}
		push(@ta, @val);
	}
	return $err;
}

sub parseTextarea2 {
	local($buf, $av1, $av2, *ta) = @_;
	my(@val, @col1, @col2, $i, $j, $k, $err);

	$err = 0;
	if ($buf) {
		if (!$av1 || !$av2) {
			$err = "parseTextarea2:$STR_ERR_BADVALUE";
			return $err;
		}
		$k = 0;
		$buf =~ s/^\s*//;
		@col1 = split(/\r\n+/, $buf);
		for ($i = 0; $i <= $#col1; $i++) {
			$col1[$i] =~ s/^\s*//;
			@col2 = split(/\s+/, $col1[$i]);
			if ($#col2 != 1) {
				$err = "parseTextarea2:$STR_ERR_BADVALUE";
				return $err;
			}
			$val[$k]->{"$av1"} = $col2[0];
			$val[$k]->{"$av2"} = $col2[1];
			$k++;
		}
		push(@ta, @val);
	}
	return $err;
}


sub parseTextareaMx {
	local($buf, *mx) = @_;
	my(@val, @col1, @col2, $i, $j, $k, $err);

	$err = 0;
	if ($buf) {
		$k = 0;
		$buf =~ s/^\s*//;
		@col1 = split(/\r\n+/, $buf);
		for ($i = 0; $i <= $#col1; $i++) {
			$col1[$i] =~ s/^\s*//;
			@col2 = split(/\s+/, $col1[$i]);
			if ($#col2 != 1) {
				$err = "parseTextareaMx:$STR_ERR_BADMX";
				return $err;
			}
			if (!&checkNum($col2[0], 0, 4294967295)) {
				$err = "$STR_ERR_BADPRIORITY($col2[0])";
				return $err;
			}
			if ($err = &namedCheckName($col2[1])) {
				$err = "$STR_ERR_BADNSNAME($col2[1])";
					return $err;
			}
			$val[$k]->{mx_priority} = $col2[0];
			$val[$k]->{mx_server} = $col2[1];
			$k++;
		}
		push(@mx, @val);
	}
	return $err;
}

#---------------------------------------------------------------------------
sub getMode {
	local($buf) = @_;
	my(@col);

	@col = split(/\./, $buf);
	if ($#col < 0) {
		return 0;
	}
	return $col[0];
}

#---------------------------------------------------------------------------
# ưå
# : 0 ()
#         1 (ư)
#----------------------------------------------------------------------------
sub checkPid {
	my($srv) = @_;
	my($result, @elm);

	$result = 0;
	@elm = ();
	if (!open(IN, "/bin/ps -ef |")) {
		goto done;
	}
	while (<IN>) {
		s/\s*$//;
		@elm = split(/\s+/);
		if ($#elm < 7) {
			next;
		}
		$elm[7] =~ s/.*\///;
		if ($elm[7] eq $srv) {
			$result = 1;
			close(IN);
			goto done;
		}
	}
	close(IN);
done:
	return($result);
}

#----------------------------------------------------------------------------
# ӥκƵư
#----------------------------------------------------------------------------
sub restartService {
	my($srv) = @_;
	my($err, $errmsg, $cmd, @msg, $i);

	$err = 0;
	$errmsg = "$STR_ERR_CANTRESTARTSV\n";
	@msg =();
	$cmd = "/etc/rc.d/init.d/$srv restart";
	if (!open(IN, "$cmd 2>&1 |")) {
		$err = $errmsg;
		goto done;
	}
	@msg = <IN>;
	close(IN);
	for ($i = 0; $i <= $#msg; $i++) {
		$msg[$i] =~ s/^\s*//;
		$msg[$i] =~ s/\s*$//;
		if ($msg[$i]) {
			$errmsg .= "$msg[$i]\n";
		}
		@col = split(/:/, $msg[$i]);
		if ($#col > 1) {
			$err = $errmsg;
		}
	}
done:
	return($err);
}

#---------------------------------------------------------------------------
# ̾Υå
#---------------------------------------------------------------------------
sub namedCheckZoneName {
	my($name) = @_;
	my($err);

	$err = 0;
	if (!defined($name) || $name eq "") {
		$err = "$STR_ERR_NODOMAIN";
		goto done;
	}
	if ($name eq ".") {
		goto done;
	}
	if ($name =~ /\.$/) {
		$err = "$STR_ERR_NONEEDDOT";
		goto done;
	}
	if ($name =~ /[^A-Za-z0-9\-\.]+/) {
		$err = "$STR_ERR_BADDOMAIN";
		goto done;
	}
done:
	return($err);
}
#---------------------------------------------------------------------------
# ̾Υå
#---------------------------------------------------------------------------
sub namedCheckName {
	my($name) = @_;
	my($err);

	$err = 0;
	if (!defined($name) || $name eq "") {
		$err = "$STR_ERR_NONAME";
		goto done;
	}
	if ($name =~ /^\./) {
		$err = "$STR_ERR_BADNAME";
		goto done;
	}
	if ($name =~ /[^A-Za-z0-9\-\.]+/) {
		$err = "$STR_ERR_BADNAME";
		goto done;
	}
done:
	return($err);
}
#---------------------------------------------------------------------------
# ɥ쥹ꥹȤΥå
#---------------------------------------------------------------------------
sub namedCheckAddrList {
	local(*col) = @_;
	my($err, $i);

	$err = 0;
	for ($i = 0; $i <= $#col; $i++) {
		if (!&ipAddrCheck($col[$i])) {
			$err = "$col[$i]";
			goto done;
		}
	}
done:
	return($err);
}

#---------------------------------------------------------------------------
# ͡ॵХꥹȤΥå
#---------------------------------------------------------------------------
sub namedCheckNsList {
	local(*col) = @_;
	my($err, $i);

	$err = 0;
	for ($i = 0; $i <= $#col; $i++) {
		if ($err = &namedCheckName($col[$i])) {
			$err = "$STR_ERR_BADNSNAME($col[$i])";
			goto done;
		}
	}
done:
	return($err);
}

#---------------------------------------------------------------------------
# ᡼륵ХꥹȤΥå
#---------------------------------------------------------------------------
sub namedCheckMxList {
	local(*col) = @_;
	my($err, $i, @col2);

	$err = 0;
	for ($i = 0; $i <= $#col; $i++) {
		$col[$i] =~ s/^\s*//;
		@col2 = split(/\s+/, $col[$i]);
		if ($#col2 != 1) {
			$err = "$STR_ERR_BADMX($col[$i])";
			goto done;
		}
		if (!&checkNum($col2[0], 0, 4294967295)) {
			$err = "$STR_ERR_BADPRIORITY($col2[0])";
			goto done;
		}
		if ($err = &namedCheckName($col2[1])) {
			$err = "$STR_ERR_BADMSNAME($col2[1])";
			goto done;
		}
	}
done:
	return($err);
}

#---------------------------------------------------------------------------
# 롼ȥ͡ॵХꥹȤΥå
#---------------------------------------------------------------------------
sub namedCheckRootNsList {
	local(*col) = @_;
	my($err, $i, @col2);

	$err = 0;
	for ($i = 0; $i <= $#col; $i++) {
		$col[$i] =~ s/^\s*//;
		@col2 = split(/\s+/, $col[$i]);
		if ($#col2 != 1) {
			$err = "$STR_ERR_BADROOTNS($col[$i])";
			goto done;
		}
		if (!&ipAddrCheck($col2[0])) {
			$err = "$STR_ERR_BADADDR($col2[0])";
			goto done;
		}
		if ($err = &namedCheckName($col2[1])) {
			$err = "$STR_ERR_BADROOTNSNAME($col2[1])";
			goto done;
		}
	}
done:
	return($err);
}

#---------------------------------------------------------------------------
# 󥿥פΥå
#---------------------------------------------------------------------------
sub namedCheckZoneType {
	my($type) = @_;
	my($err);

	$err = 0;
	if ($ztype ne "master"
		&& $ztype ne "slave"
		&& $ztype ne "stub"
		&& $ztype ne "forward"
		&& $ztype ne "hint") {
		$err = "$STR_ERR_BADTYPE";
	}
	return($err);
}
#---------------------------------------------------------------------------
# ɥᥤ̾åΥפΥå
#---------------------------------------------------------------------------
sub namedCheckCheckNamesType {
	my($val) = @_;
	my($err);

	$err = 0;
	if ($val && $val ne "master"
		&& $val ne "slave"
		&& $val ne "response") {
		$err = "$STR_ERR_BADCHECKNAMESTYPE";
	}
	return($err);
}

#---------------------------------------------------------------------------
# ɥᥤ̾åˡΥå
#---------------------------------------------------------------------------
sub namedCheckCheckNamesMethod {
	my($val) = @_;
	my($err);

	$err = 0;
	if ($val && $val ne "warn"
		&& $val ne "fail"
		&& $val ne "ignore") {
		$err = "$STR_ERR_BADCHECKNAMESMETHOD";
	}
	return($err);
}

#---------------------------------------------------------------------------
# yes/noå
#---------------------------------------------------------------------------
sub namedCheckYesNo {
	my($val) = @_;
	my($err);

	$err = 0;
	if ($val && $val ne "yes"
		&& $val ne "no") {
		$err = "$STR_ERR_BADVALUE";
	}
	return($err);
}

#---------------------------------------------------------------------------
# first/onlyå
#---------------------------------------------------------------------------
sub namedCheckOnlyFirst {
	my($val) = @_;
	my($err);

	$err = 0;
	if ($val && $val ne "only"
		&& $val ne "first") {
		$err = "$STR_ERR_BADVALUE";
	}
	return($err);
}

#---------------------------------------------------------------------------

1;

